class ColorsClass:  # Colors required for making look **good**
    def __init__(self):
        self.HEADER = '\033[95m'
        self.OKBLUE = '\033[94m'
        self.OKCYAN = '\033[96m'
        self.OKGREEN = '\033[92m'
        self.WARNING = '\033[93m'
        self.FAIL = '\033[91m'
        self.ENDC = '\033[0m'  # in order to stop colors from filling the entire terminal.
        self.BOLD = '\033[1m'
        self.UNDERLINE = '\033[4m'

    def clear(self):
        self.HEADER = ''
        self.OKBLUE = ''
        self.OKCYAN = ''
        self.OKGREEN = ''
        self.WARNING = ''
        self.FAIL = ''
        self.ENDC = ''
        self.BOLD = ''
        self.UNDERLINE = ''


Colors = ColorsClass()


# Custom function not found exception, used for missing functions
class FunctionNotFound(BaseException):
    def __init__(self, text=None):
        if text is None:
            print(f"{Colors.FAIL}ERR: Failed to find a function inside this class! Please enter a valid class.\nIf this class has more than 1 function, (excluding __ functions) then use the \"optionalFunc\" parameter.{Colors.ENDC}")  # NOQA:E501
        else:
            print(f"{Colors.FAIL}ERR: {text}{Colors.ENDC}")


# Custom Not Enough Testcases exception, used when len(testcases) == 0
class NotEnoughTestcases(BaseException):
    def __init__(self, text=None):
        if text is None:
            print(f"{Colors.FAIL}Testcases length is too small!{Colors.ENDC}")
        else:
            print(f"{Colors.FAIL}ERR: {text}{Colors.ENDC}")


def fillFunction():
    raise FunctionNotFound("We failed to find a function inside this class provided.\n"
                           "Please enter a valid class. If this class has more than ONE function inside "
                           "(excluding __ functions), then use the \"optionalFunc\" parameter to enter the function.")



from .supportClasses import ColorsClass, fillFunction, NotEnoughTestcases
Colors = ColorsClass()
useClass = False


# simple driver that runs testcases
def driver(solution: type.__class__, testcases: dict, optionalFunc=None, colorless=False) -> None:
    global useClass
    assert isinstance(testcases, dict), "testcases must be a dictionary that is setup like this: {testcase:answer}!"

    if colorless:
        Colors.clear()
    if len(testcases) < 1:
        raise NotEnoughTestcases("testcases must contain at least one testcase!")
    if optionalFunc is None:
        functions = [attr for attr in dir(solution) if callable(getattr(solution, attr)) and attr.startswith('__') is False]  # NOQA:E501
        if len(functions) == 1:
            func = getattr(solution, functions[0])
        else:
            func = fillFunction()
    else:
        func = optionalFunc

    numberCorrect = 0
    print(f"Using function: {solution.__class__.__name__}.{func.__name__}()")
    for testNum, testcase in enumerate(testcases):
        print(f"Test {testNum + 1}/{len(testcases)}: {testcase}")
        val = func(testcase)
        print(f"Test {testNum + 1}/{len(testcases)} returned: {val}.")
        if testcases[testcase] == val:
            print(f"{Colors.OKCYAN}Test {testNum + 1}/{len(testcases)} was SUCCESSFUL.{Colors.ENDC}")
            numberCorrect += 1
        else:
            print(f"{Colors.WARNING}Test {testNum + 1}/{len(testcases)} FAILED.{Colors.ENDC}")
        print("\n\n")

    print(f"{Colors.BOLD}")  # make everything from the next text bold.
    if numberCorrect == len(testcases):
        print(f"{Colors.OKGREEN}\n\n*****\nALL SUCCESSFUL\n*****{Colors.ENDC}")
    elif numberCorrect > len(testcases) - 2:
        print(f"{Colors.WARNING}\n\n*****\n{numberCorrect}/{len(testcases)} Successful\n*****{Colors.ENDC}")
    else:
        print(f"{Colors.FAIL}\n\n*****\n{numberCorrect}/{len(testcases)} Successful\n*****{Colors.ENDC}")



from setuptools import setup, find_packages

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Education',
    'Operating System :: OS Independent',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3'
]

setup(
    name='leetcodeDriverPY',
    version='1.0.2',
    description='A simple library to help people run Leetcode testcases without the Leetcode online IDE.',
    long_description_content_type='text/markdown',
    long_description=open('README.md').read() + '\n\n' + open('CHANGELOG.txt').read(),
    url='https://github.com/Duve3/leetcodeDriverPY',
    bug_tracker='https://github.com/Duve3/leetcodeDriverPY/issues',
    author='Duve3',
    author_email='',
    license='MIT',
    classifiers=classifiers,
    keywords='leetcode',
    packages=find_packages(),
    install_requires=['']
)
